//      flashprog.h
//      
//      Copyright 2012 swausd
//      
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.

#ifndef FLASHPROG_H
#define FLASHPROG_H

#include "types.h"

// Flasch Commands for "Intel 3 Volt Advanced Boot Block Flash Memory Family"
#define FLASH_CMD_READ_ARRAY     0xff
#define FLASH_CMD_PROGRAM_SETUP  0x40
#define FLASH_CMD_ERASE_SETUP    0x20
#define FLASH_CMD_ERASE_CONFIRM  0xd0
#define FLASH_CMD_RESUME         0xd0
#define FLASH_CMD_UNLOCK_BLOCK   0xd0
#define FLASH_CMD_SUSPEND        0xb0
#define FLASH_CMD_READ_STATUS    0x70
#define FLASH_CMD_CLEAR_STATUS   0x50
#define FLASH_CMD_READ_CONFIG    0x90
#define FLASH_CMD_CONFIG_SETUP   0x60
#define FLASH_CMD_LOCK_BLOCK     0x01
#define FLASH_CMD_LOCKDOWN_BLOCK 0x2f
#define FLASH_CMD_READ_QUERY     0x98
#define FLASH_CMD_PROT_PRG_SETUP 0xc0
#define FLASH_CMD_ALT_PRG_SETUP  0x10

#define FLASH_STATUS_WSMS        0b10000000  // Write Machine Status
#define FLASH_STATUS_ESS         0b01000000  // Erase Suspend Status
#define FLASH_STATUS_ES          0b00100000  // Erase Status
#define FLASH_STATUS_PS          0b00010000  // Program Status
#define FLASH_STATUS_VPPS        0b00001000  // VPP Status
#define FLASH_STATUS_PSS         0b00000100  // Program Suspend Status
#define FLASH_STATUS_BLS         0b00000010  // Block Lock Status

// user space error codes
#define FLASH_NO_ERROR           0
#define FLASH_PARAMETER_ERROR    1
#define FLASH_ERASE_ERROR        2
#define FLASH_WRITE_ERROR        3
#define FLASH_BLOCK_LOCKED_ERROR 4
#define FLASH_CMD_SEQUENCE_ERROR 5

void flashSetBaseAdr(uint16_t *adr);
uint8_t flashWaitUntilReady(void);
uint8_t flashGetStatus(void);
uint16_t flashGetWord(uint32_t offset);
void flashSetWord(uint16_t data, uint32_t offset);
void flashSetCommand(uint16_t command, uint32_t offset);
uint8_t flashEraseBlock(uint32_t offset);
uint8_t flashWriteWord(uint32_t offset, uint16_t *data, uint32_t wordCount);
void flashNormalOperation(void);
void flashUnlockBlock(uint32_t offset);

#endif /* FLASHPROG_H */
