//      main.c
//      
//      Copyright 2012 swausd
//      
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//
//      -----
//
//      This is a test framework for programming the flash memory in the
//      Vivonic Fitness Planner, a PDA sytle device based on a Freescale
//      68EZ328 Dragonball cpu.
//      Flash memory is a Intel TE28F160-C3TA90
//      16 MBit organized in 16 Bit Words

#include "types.h"
#include "serio.h"
#include "flashprog.h"


#define FLASH_BASE_ADR (uint16_t *) 0x800000


int main(int argc, char **argv)
{
   uint16_t data[128];
   uint8_t error;
   
   // fill buffer with test pattern to write into flash
   for (int i = 0; i < 128; i++)
   {
      data[i] = 0xaa55;
   }
 
   // dump "before" conten of flash memory
	putStringToUart("before write image\r\n");
   putDumpToUart((uint8_t *) 0x8b0000, 0x8b0000, 256*3);
 
   // setup base address for flash io subroutines
   flashSetBaseAdr(FLASH_BASE_ADR);
 
   // flash memory is locked, so unlock first
   flashUnlockBlock(0xb0100);
   
   // write test pattern to flash
   error = flashWriteWord(0xb0100, data, 128);
   
   // output error code so uart
	putStringToUart("flashWriteWord return code: ");
   putHexByteToUart(error);
   putCrNlToUart();
   
   // reset flash memory to normal read operation
   flashNormalOperation();
   
   // dump "after" content of flash memory to check
	putStringToUart("after write image\r\n");
   putDumpToUart((uint8_t *) 0x8b0000, 0x8b0000, 256*3);
 
   // flash memory is locked, so unlock first
   flashUnlockBlock(0xb0100);

   // erase one block of flash memory
   error = flashEraseBlock(0x0b0100);
 
   // output error code so uart
	putStringToUart("flashEraseBlock return code: ");
   putHexByteToUart(error);
   putCrNlToUart();
   
   // reset flash memory to normal read operation
   flashNormalOperation();
   
   // dump "after" content of flash memory to check
	putStringToUart("after erase image\r\n");
   putDumpToUart((uint8_t *) 0x8b0000, 0x8b0000, 256*3);

	return 0;
}
