//      serio.c
//      
//      Copyright 2012 Siegfried Wünsche <swuensche@MacDebian>
//      
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//
//      -----
//
//      Some utility subroutines for I/O to the UART of the
//      Vivonic Fitness Planner, a PDA sytle device based on a Freescale
//      68EZ328 Dragonball cpu.
//      Note: serial output uses busy wait.

#include "serio.h"


// Output a single character

void putCharToUart(char c)
{
   while (UTX & UTX_BUSY)
      ;
   UTX_TXDATA = c;
}


// Output a string

void putStringToUart(char *s)
{
   while (*s)
   {
      putCharToUart(*s++);
   }
}


// Output carriage return and line feed

void putCrNlToUart(void)
{
   putStringToUart("\r\n");
}


// Output a single byte as hex coded string

void putHexByteToUart(uint8_t c)
{
	static char hex[17]="0123456789ABCDEF";

	putCharToUart(hex[c >> 4]);
	putCharToUart(hex[c & 0x0f]);
}


// Output a single 16 bit word as hex coded string

void putHexWordToUart(uint16_t c)
{
	putHexByteToUart(c >> 8);
	putHexByteToUart(c & 0x00ff);
}


// Output a 32 bit address in [] as hex coded string

void putAddressToUart(uint32_t adr)
{
   putCharToUart('[');
   putHexWordToUart(adr >> 16);
   putHexWordToUart(adr &  0xffff);
   putStringToUart("] ");
}


// Output a formatted memory dump using *data as start address.
// offset can be used to "transformed" the output to a different
// address space.

void putDumpToUart(uint8_t *data, uint32_t offset, uint32_t byteCount)
{
   uint32_t adr = offset & 0xfffffff0;
   uint32_t idx = 0;
   
   if (data != NULL)
   {
      if (adr < offset)
      {
         putAddressToUart(adr);
      
         while (adr < offset)
         {
            adr++;
            putStringToUart("   ");
         }
      }
         
      do
      {
         if ((adr & 0xf) == 0)
            putAddressToUart(adr);

         do
         {
            putCharToUart(' ');
            putHexByteToUart(data[idx++]);
            adr++;
         }
         while (((adr & 0xf) != 0) && (idx < byteCount));
         
            putCrNlToUart();
      }
      while (idx < byteCount);
   }

   putCrNlToUart();
}
